import webbrowser
from textual import on
from textual.app import App, ComposeResult, events
from textual.screen import Screen
from textual.widgets import ContentSwitcher
from smassh.ui.events import SetScreen, ShowResults
from smassh.ui.widgets import *  # noqa
from smassh.ui.screens import *  # noqa
from smassh.ui.widgets.palette.palette_list import ApplyLanguage, ApplyTheme
from smassh.ui.widgets.palette import LanguagePalette, ThemePalette
from smassh.src import config_parser, generate_theme_file, data_parser
from smassh.ui.widgets import Space, Ticker
from smassh.ui.screens.confirm import ConfirmScreen


class MainScreen(Screen):
    """
    Main Screen which renders all the first visible option when app starts
    """

    DEFAULT_CSS = """
    MainScreen {
        layout: grid;
        grid-size: 1 2;
        grid-rows: 5 1fr;
    }
    """

    def compose(self) -> ComposeResult:
        yield Header()
        yield ContentSwitcher(
            TypingScreen(id="typing"),
            AboutScreen(id="about"),
            SettingsScreen(id="settings"),
            HelpScreen(id="help"),
            ResultScreen(id="result"),
            # initial screen
            initial="typing",
        )

    @on(SetScreen)
    def screen_change(self, event: SetScreen) -> None:
        """
        Change BaseWidget in the main screen depending on name

        Args:
            event (SetScreen): event with screen name
        """

        self.query_one(ContentSwitcher).current = event.screen_name
        self.query_one(Header).set_active(event.screen_name)

    @on(ShowResults)
    def show_results(self, event: ShowResults) -> None:
        """
        Triggered when typing is finished

        Args:
            event (ShowResults): Event containing stats and if it the typing was a failed attempt
        """

        # reset all watching timers
        space = self.query_one(Space)
        space.check_timer.pause()
        space.tracker.stats.finish()
        self.query_one(Ticker).reset()

        self.query_one(ContentSwitcher).current = "result"
        self.query_one(ResultScreen).set_results(event.stats)

        data_parser.add_stats(event.stats, event.failed)

    async def handle_key(self, event: events.Key) -> None:
        visible = self.query_one(ContentSwitcher).visible_content
        if visible:
            await visible.handle_key(event)


class Smassh(App):
    CSS_PATH = "css/styles.tcss"
    SCREENS = {
        "main": MainScreen(),
        "theme": ThemePaletteScreen(),
        "language": LanguagePaletteScreen(),
        "confirm": ConfirmScreen(),
    }

    def __init__(self, *args, **kwargs) -> None:
        self.action_theme(config_parser.get("theme"))
        super().__init__(*args, **kwargs, watch_css=True)

    async def _on_css_change(self) -> None:
        await super()._on_css_change()
        self.refresh_css()

    async def on_mount(self) -> None:
        self.push_screen("main")

    @on(ApplyLanguage)
    def apply_language(self, event: ApplyLanguage) -> None:
        config_parser.set("language", event.value)
        self.SCREENS["main"].query_one(LanguagePalette).refresh()
        self.SCREENS["main"].query_one(Space).reset()

    @on(ApplyTheme)
    def apply_theme(self, event: ApplyTheme) -> None:
        self.action_theme(event.value)
        config_parser.set("theme", event.value)
        self.SCREENS["main"].query_one(ThemePalette).refresh()

    def action_star(self) -> None:
        webbrowser.open("https://github.com/kraanzu/smassh")

    def action_theme(self, theme: str) -> None:
        generate_theme_file(theme)



import operator
import textwrap
from json import load
from itertools import accumulate
from collections.abc import Callable
from typing import List
from smassh.src.parser import config_parser
from random import randint, choice, sample


PUNCS = ",.;?!"
GeneratorFunc = Callable[..., str]


def numbers(func: GeneratorFunc) -> GeneratorFunc:
    def wrapper(*args, **kwargs):
        paragraph = func(*args, **kwargs)

        if not config_parser.get("numbers"):
            return paragraph

        words = paragraph.split()
        total_words = len(words)
        words_to_insert = total_words // 5
        positions = sample(range(total_words), words_to_insert)
        positions.sort()

        for position in positions:
            numbers = [str(randint(10**i, 10 ** (i + 1))) for i in range(4)]
            words[position] = choice(numbers)

        return " ".join(words)

    return wrapper


def punctuations(func: GeneratorFunc) -> GeneratorFunc:
    def wrapper(*args, **kwargs):
        paragraph = func(*args, **kwargs)

        if not config_parser.get("punctuations"):
            return paragraph

        words = paragraph.split()
        new_paragraph = []

        for word in words:
            i = randint(0, 20)
            if i == 0:
                word = word + choice(PUNCS)
            elif i == 1:
                word = f"'{word}'"
            elif i == 2:
                word = f'"{word}"'
            elif i == 3:
                word = f"({word})"

            new_paragraph.append(word)

        return " ".join(new_paragraph)

    return wrapper


def capitalization(func: GeneratorFunc) -> GeneratorFunc:
    def wrapper(*args, **kwargs):
        paragraph = func(*args, **kwargs)

        if config_parser.get("capitalization_mode") == "off":
            return paragraph

        def convert(word: str) -> str:
            if config_parser.get("capitalization_mode") == "on":
                return word[0].upper() + word[1:]

            return "".join([i.upper() if randint(0, 1) else i for i in word])

        words = paragraph.split()
        new_paragraph = []

        for word in words:
            i = randint(0, 2)
            if i == 0:
                word = convert(word)

            new_paragraph.append(word)

        return " ".join(new_paragraph)

    return wrapper


class Generator:
    """
    Generator class to generate paragraphs for typing tests
    """

    def __init__(self) -> None:
        self.settings = {}

    def get_words(self, language: str) -> List[str]:
        from smassh.src.parser.data_parser import DataParser

        path = DataParser.lang_path / f"{language}.json"
        with open(path) as f:
            return load(f)["words"]

    @capitalization
    @punctuations
    @numbers
    def generate(self, language: str, size: int) -> str:
        words = self.get_words(language)

        if size > len(words):
            q = size // len(words)
            words = words * q + words[: size % len(words)]

        return " ".join(sample(words, size))

    def get_newlines(self, paragraph: str, width: int) -> List[int]:
        lines = textwrap.wrap(paragraph, width)
        return list(
            accumulate(
                [len(line) + 1 for line in lines],
                operator.add,
            )
        )


master_generator = Generator()



from .value import Value, ValueLabel, ValueContainer
from .result_strip import ResultStrip, NextTest, RepeatTest

__all__ = [
    "Value",
    "ValueLabel",
    "ResultStrip",
    "NextTest",
    "RepeatTest",
    "ValueContainer",
]



from rich import print
import requests
import appdirs
from typing import Optional
from pathlib import Path

LANGUAGE_PACK_DIR = Path(appdirs.user_data_dir("smassh")) / "languages"


class AddLanguage:
    """
    Plugin to add new languages to smassh
    """

    def __init__(self, silent: bool = False) -> None:
        self.silent = silent

    def log(self, message: str, color: str = "green") -> None:
        """Logs a message to the console"""
        if not self.silent:
            print(f"=>[bold {color}] {message}[/bold {color}]")

    def get_pack(self, name: str) -> Optional[str]:
        """Checks if a language pack exists. If found, it returns its contents otherwise it returns None"""

        uri = f"https://raw.githubusercontent.com/monkeytypegame/monkeytype/master/frontend/static/languages/{name}.json"
        req = requests.get(uri)

        return req.text if (req.status_code == 200) else None

    def add(self, name: str) -> None:
        """Downloads a new language for smassh"""

        if not LANGUAGE_PACK_DIR.exists():
            LANGUAGE_PACK_DIR.mkdir()

        self.log("Checking if language pack exists...")

        pack = self.get_pack(name)

        if pack is None:
            return self.log("Language pack doesnt exist!", "red")

        LANGUAGE_FILE = LANGUAGE_PACK_DIR / f"{name}.json"

        if LANGUAGE_FILE.exists():
            return self.log("Language pack already exists", "green")

        self.log("Downloading language pack...")

        with open(LANGUAGE_FILE, "w") as f:
            f.write(pack)

        self.log("Successfully downloaded the language pack!")



from pathlib import Path
from typing import List
from .parser import Parser
import appdirs

DEFAULTS = {
    "difficulty": "normal",
    "blind_mode": False,
    "min_speed": 0,
    "min_accuracy": 0,
    "min_burst": 0,
    "force_correct": False,
    "confidence_mode": "off",
    "capitalization_mode": "off",
    "cursor_buddy_speed": 0,
    "tab_reset": False,
    "language": "english",
    "numbers": False,
    "punctuations": False,
    "mode": "words",
    "words_count": 30,
    "time_count": 30,
    "caret_style": "block",
    "writing mode": "words",
    "theme": "nord",
}


class ConfigParser(Parser):
    """
    Inherited from `Parser` class to manage config
    """

    _root_dir = Path(__file__).parent.parent.parent
    config_path = Path(appdirs.user_config_dir("smassh"))
    DEFAULT_CONFIG = DEFAULTS

    @property
    def configured_languages(self) -> List[str]:
        from smassh.src.parser.data_parser import DataParser

        words_dir = DataParser.lang_path

        languages = [
            file_obj.stem
            for file_obj in words_dir.iterdir()
            if file_obj.suffix == ".json"
        ]
        return languages

    @property
    def configured_themes(self) -> List[str]:
        themes_dir = self._root_dir / "ui" / "css" / "themes"
        themes = [
            file_obj.stem
            for file_obj in themes_dir.iterdir()
            if file_obj.suffix == ".tcss"
        ]
        return themes

    def toggle_numbers(self) -> None:
        numbers = self.get("numbers")
        self.set("numbers", not numbers)

    def toggle_punctuations(self) -> None:
        punctuations = self.get("punctuations")
        self.set("punctuations", not punctuations)

    def toggle_mode(self) -> None:
        mode = self.get("mode")
        mode = "words" if mode == "time" else "time"
        self.set("mode", mode)

    @property
    def theme(self) -> str:
        return self.get("theme") or "nord"

    @theme.setter
    def theme(self, theme: str) -> None:
        self.set("theme", theme)


config_parser = ConfigParser()



from .option import Option, NumberScroll
from .settings_options import menu
from .separator import SettingSeparator
from .strip import SettingStrip, SettingStripItem
from ..typing.config_strip import Bracket

__all__ = [
    "Option",
    "NumberScroll",
    "menu",
    "SettingSeparator",
    "SettingStrip",
    "SettingStripItem",
    "Bracket",
]



from rich.console import RenderableType
from textual.app import ComposeResult
from textual.widgets import Label, Static
from smassh.ui.events import SetScreen
from smassh.ui.widgets import Space


class ResultStripItem(Label):
    """
    A strip widget item containg action after typing is finished
    """

    DEFAULT_CSS = """
    ResultStripItem {
        padding: 0 3;
    }
    """

    icon: str
    help: str

    def on_mount(self) -> None:
        self.tooltip = self.help

    def render(self) -> RenderableType:
        return self.icon


class NextTest(ResultStripItem):
    icon = ""
    help = "Next test"

    def on_click(self, _) -> None:
        self.screen.query_one(Space).reset()
        self.post_message(SetScreen("typing"))


class RepeatTest(ResultStripItem):
    icon = ""
    help = "Repeat test"

    def on_click(self, _) -> None:
        self.screen.query_one(Space).restart(force=True)
        self.post_message(SetScreen("typing"))


class ResultStrip(Static):
    """
    Strip widget that contain all the available actions on result screen
    """

    DEFAULT_CSS = """
    ResultStrip {
        layout: horizontal;
        align: center middle;
    }
    """

    def compose(self) -> ComposeResult:
        yield NextTest()
        yield RepeatTest()



from typing import Optional
from textual.app import events
from textual.widget import Widget
from smassh.ui.events import SetScreen


class BaseWindow(Widget):
    """
    Base Window widget for content switcher
    """

    DEFAULT_CSS = """
    BaseWindow {
        layout: vertical;
        overflow-y: auto;
        height: 1fr;
        scrollbar-size: 1 1;
    }
    """

    async def handle_key(self, event: events.Key) -> Optional[bool]:
        if event.key == "escape":
            return self.post_message(SetScreen("typing"))



from typing import List, Tuple
from rich.console import RenderableType
from rich.table import box
from rich.text import Text
from textual.widget import Widget
from textual.app import ComposeResult
from smassh.src.help_menu import SETTINGS_KEYBINDS, TYPING_KEYBINDS, GENERAL_KEYBINDS
from smassh.ui.widgets.base_window import BaseWindow


class Table(Widget):
    """
    Table widget to show keybindings
    """

    DEFAULT_CSS = """
    Table {
        height: auto;
        content-align: center middle;
        align: center middle;
        margin: 1 3;
    }
    """

    COMPONENT_CLASSES = {"--header", "--key", "--action"}

    def __init__(self, title: str, keys: List[Tuple[str, str]] = []) -> None:
        super().__init__()
        self.title = title
        self.keys = keys

    def render(self) -> RenderableType:
        from rich.table import Table

        table_header = self.get_component_rich_style("--header")
        table_key = self.get_component_rich_style("--key")
        table_action = self.get_component_rich_style("--action")

        table = Table(
            expand=True,
            show_header=False,
            padding=(0, 0),
            box=box.SIMPLE,
            title=Text(
                f" 󰌌 Keybinds for {self.title}",
                style=table_header,
                justify="left",
            ),
        )
        table.add_column(Text("Key"), style=table_key, ratio=1)
        table.add_column("", width=5)
        table.add_column(Text("Action"), style=table_action, ratio=4)
        for key, description in self.keys:
            table.add_row(key, "", description)

        return table


class HelpScreen(BaseWindow):
    """
    Help Screen to show help tables for various screens
    """

    def compose(self) -> ComposeResult:
        yield Table("General Stuff", GENERAL_KEYBINDS)
        yield Table("Typing Screen", TYPING_KEYBINDS)
        yield Table("Settings Screen", SETTINGS_KEYBINDS)



from rich.console import RenderableType
from smassh.src import config_parser
from textual.widget import Widget


class Palette(Widget):
    """
    Palette Widget to show current set palette option
    """

    DEFAULT_CSS = """
    Palette {
        margin: 0 3;
        width: auto;
        height: 1;
    }
    """

    screen_name: str
    config_name: str
    icon: str

    @property
    def current(self) -> str:
        return config_parser.get(self.config_name)

    async def on_click(self, _) -> None:
        await self.app.push_screen(self.screen_name)
        self.app.refresh_css(animate=False)

    def render(self) -> RenderableType:
        return f"{self.icon} {self.current.replace('_', ' ')}"



from .parser import config_parser, data_parser
from .tracker import Tracker, Cursor
from .figlet import generate_figlet
from .css_generator import generate_theme_file
from .generator import master_generator
from .stats_tracker import StatsTracker
from .buddy import Buddy


__all__ = [
    "config_parser",
    "data_parser",
    "Tracker",
    "Cursor",
    "generate_figlet",
    "generate_theme_file",
    "master_generator",
    "StatsTracker",
    "Buddy",
]



import os
from pathlib import Path

CSS = ""
EXTENSION = ".tcss"


def load_folder(folder: Path) -> str:
    """
    Load all css files in a folder and return them as a single string
    """

    css = ""

    for file in os.listdir(folder):
        if not file.endswith(EXTENSION):
            continue

        with open(os.path.join(folder, file), "r") as css_file:
            css += css_file.read()

    return css


css_folder = Path(__file__).parent / "css"
theme_folder = css_folder / "themes"


CSS += load_folder(css_folder)
CSS += load_folder(theme_folder)



GENERAL_KEYBINDS = [
    ("ctrl+c", "Quit App"),
    ("ctrl+s", "Open Settings"),
]

TYPING_KEYBINDS = [
    ("escape", "Reset typing"),
    ("ctrl+n", "Toggle numbers"),
    ("ctrl+p", "Toggle punctuations"),
    ("ctrl+w", "Delete a whole word"),
    ("ctrl+l", "Opens language menu"),
    ("ctrl+t", "Opens theme menu"),
]

SETTINGS_KEYBINDS = [
    ("j/down", "Move to next setting"),
    ("k/up", "Move to previous setting"),
    ("tab", "Move to next option"),
    ("shift+tab", "Move to previous option"),
]



from .typing import TypingScreen
from .help import HelpScreen
from .about import AboutScreen
from .settings import SettingsScreen
from .result import ResultScreen
from .palette import LanguagePaletteScreen, ThemePaletteScreen

__all__ = [
    "TypingScreen",
    "ResultScreen",
    "HelpScreen",
    "AboutScreen",
    "SettingsScreen",
    "LanguagePaletteScreen",
    "ThemePaletteScreen",
]



from typing import Optional
from rich.console import RenderableType
from textual.reactive import reactive
from textual.widget import Widget
from smassh.src import generate_figlet
from smassh.ui.events import SetScreen


class NavItemBase(Widget):
    """
    Base Widget for Header NavItems
    """

    DEFAULT_CSS = """
    NavItemBase {
        content-align: center middle;
        height: auto;
        width: auto;
        padding: 0 1;
    }
    """

    def __init__(self, text: str, screen_name: Optional[str] = None) -> None:
        super().__init__()
        self.text = text
        self.screen_name = screen_name

    def on_click(self) -> None:
        if self.screen_name:
            self.post_message(SetScreen(self.screen_name))

    def render(self) -> RenderableType:
        return self.text


class Banner(NavItemBase):
    """
    Text Widget to render text in a bigger font
    """

    DEFAULT_CSS = """
    Banner {
        height: 100%;
    }
    """

    is_tall = reactive(True, layout=True, always_update=True)

    def watch_is_tall(self, value: bool) -> None:
        self.styles.height = "5" if value else "3"

    def render(self) -> RenderableType:
        return generate_figlet(self.text) if self.is_tall else self.text.upper()


class NavItem(NavItemBase):
    """
    Just a label widget with a callback
    """



from typing import Literal
from rich.console import RenderableType
from textual.app import ComposeResult
from textual.binding import Binding
from textual.screen import Screen
from textual.widget import Widget


class ConfirmButton(Widget):
    DEFAULT_CSS = """
    ConfirmButton {
        padding: 1;
        height: 3;
        content-align: center middle;
    }
    """

    def __init__(
        self,
        label: str,
        button_type: Literal["ok", "cancel"],
        *args,
        **kwargs,
    ) -> None:
        classes = "ok-btn" if button_type == "ok" else "cancel-btn"
        super().__init__(classes=classes, *args, **kwargs)
        self.label = label
        self.button_type = button_type

    def on_click(self) -> None:
        return self.dismiss()

    def dismiss(self) -> None:
        return self.screen.dismiss(self.button_type == "ok")

    def render(self) -> RenderableType:
        return self.label


class MessageBox(Widget):
    DEFAULT_CSS = """
    MessageBox {
        content-align: center middle;
        column-span: 2;
        text-style: bold italic;
    }
    """

    def render(self) -> RenderableType:
        return "Are you sure?\nThis action cannot be undone."


class MessageDialogue(Widget):

    DEFAULT_CSS = """
    MessageDialogue {
        layout: grid;
        width: 50;
        height: auto;
        background: black;
        grid-size: 2 2;
        grid-rows: 4 3;
        grid-columns: 1fr 1fr;
        border: solid red;
    }
    """

    def compose(self) -> ComposeResult:
        self.ok = ConfirmButton("[O]K", button_type="ok")
        self.cancel = ConfirmButton("[C]ANCEL", button_type="cancel")

        yield MessageBox()
        yield self.ok
        yield self.cancel


class ConfirmScreen(Screen):
    DEFAULT_CSS = """
    ConfirmScreen {
        align: center middle;
    }
    """

    BINDINGS = [
        Binding("escape", "app.pop_screen"),
    ]

    def compose(self) -> ComposeResult:
        yield MessageDialogue()

    def key_o(self):
        self.query_one(MessageDialogue).ok.dismiss()

    def key_c(self):
        self.query_one(MessageDialogue).cancel.dismiss()



from dataclasses import dataclass
from typing import Callable, Optional
from smassh.src.parser import config_parser
from .stats_tracker import StatsTracker, CheckPoint, Match

TrackerFunc = Callable[..., Optional["Cursor"]]


def force_correct(func: TrackerFunc) -> TrackerFunc:
    def wrapper(tracker: "Tracker", key: str) -> Optional[Cursor]:
        setting = config_parser.get("force_correct") == "on"
        if not setting or tracker.paragraph[tracker.cursor_pos] == key:
            return func(tracker, key)

    return wrapper


def confidence_mode(func: TrackerFunc) -> TrackerFunc:
    def wrapper(tracker: "Tracker", *args, **kwargs) -> Optional[Cursor]:
        setting = config_parser.get("confidence_mode")
        if setting == "max":
            return

        if (
            setting == "on"
            and tracker.cursor_pos
            and tracker.paragraph[tracker.cursor_pos - 1] == " "
        ):
            return

        result = func(tracker, *args, **kwargs)
        return result

    return wrapper


def difficulty(func: TrackerFunc) -> TrackerFunc:
    def wrapper(tracker: "Tracker", key: str) -> Optional[Cursor]:
        setting = config_parser.get("difficulty")

        if setting == "master" and tracker.paragraph[tracker.cursor_pos] != key:
            return

        if (
            setting == "expert"
            and tracker.paragraph[tracker.cursor_pos] == " "
            and tracker.stats.last_word_accuracy < 100
        ):
            return

        return func(tracker, key)

    return wrapper


@dataclass
class Cursor:
    """
    Cursor class to maintain record of checkpoints
    """

    old: int
    new: int
    correct: bool
    letter: str = ""

    def to_checkpoint(self) -> CheckPoint:
        if self.new > self.old:
            return CheckPoint(
                self.letter, self.new, Match.MATCH if self.correct else Match.MISMATCH
            )

        return CheckPoint(self.letter, self.new, Match.BACKSPACE)


class Tracker:
    """
    Tracker class to track keypresses on typing test
    """

    def __init__(self, paragraph: str) -> None:
        self.reset(paragraph)

    def reset(self, paragraph: str) -> None:
        self.paragraph = paragraph
        self.stats = StatsTracker()
        self.cursor_pos = 0

    def keypress(self, key: str) -> Optional[Cursor]:
        res = None

        if key == "backspace":
            res = self.handle_delete_letter()

        elif key == "ctrl+w":
            res = self.handle_delete_word()

        elif len(key) == 1:
            res = self.handle_letter(key)

        if res:
            self.stats.add_checkpoint(res.to_checkpoint())
            return res

    @confidence_mode
    def handle_delete_letter(self) -> Optional[Cursor]:
        old = self.cursor_pos

        if self.cursor_pos == 0:
            return

        self.cursor_pos -= 1
        return Cursor(old, self.cursor_pos, True)

    @confidence_mode
    def handle_delete_word(self) -> Optional[Cursor]:
        old = self.cursor_pos

        if self.cursor_pos == 0:
            return

        # incase it's the start of a word
        if self.cursor_pos:
            self.cursor_pos -= 1

        while self.cursor_pos > 0 and self.paragraph[self.cursor_pos - 1] != " ":
            self.cursor_pos -= 1

        return Cursor(old, self.cursor_pos, True)

    @difficulty
    @force_correct
    def handle_letter(self, key: str) -> Optional[Cursor]:
        if self.cursor_pos >= len(self.paragraph):
            return

        old = self.cursor_pos
        correct = key == self.paragraph[old]
        self.cursor_pos += 1
        return Cursor(old, self.cursor_pos, correct, self.paragraph[old])



import os
from textual.app import ComposeResult
from textual.containers import Horizontal
from textual.widget import Widget
from textual.widgets import Static

from smassh.ui.widgets.label import Banner, NavItem


def get_username() -> str:
    try:
        username = os.getlogin()
    except OSError:
        uid = os.getuid()
        import pwd

        username = pwd.getpwuid(uid).pw_name

    return username


class Header(Widget):
    """
    Header which forms the top banner of the app
    """

    DEFAULT_CSS = """
    Header {
        layout: grid;
        grid-size: 4 1;
        grid-rows: 5;
        grid-columns: 1fr 15 8fr 1fr;
    }

    Header > Horizontal {
        align: left middle;
        height: 100%;
    }
    """

    def on_resize(self) -> None:
        # XXX: Why use screen size and not widget size?
        # Ans: Because this can change the widget size (be it header or container)
        #      which will trigger this method again, causing an infinite loop
        height = self.screen.size.height

        # NOTE: This seems like a good ratio (5:30) to enable/disable tall mode
        if height < 30:
            self.disable_tall_mode()
        else:
            self.enable_tall_mode()

        self.refresh(layout=True)

    def enable_tall_mode(self) -> None:
        height = self.size.height
        if height == 5:
            return

        self.screen.styles.grid_rows = "5 1fr"
        self.query_one(Banner).is_tall = True
        self.query_one("Header > Horizontal").styles.height = "5"

    def disable_tall_mode(self) -> None:
        height = self.size.height
        if height != 5:
            return

        self.screen.styles.grid_rows = "3 1fr"
        self.query_one(Banner).is_tall = False
        self.query_one("Header > Horizontal").styles.height = "3"

    def set_active(self, name: str) -> None:
        for i in self.query(NavItem):
            i.set_class(i.screen_name == name, "active")

    def compose(self) -> ComposeResult:
        yield Static()
        yield Banner("smassh")

        with Horizontal():
            home = NavItem("󰌌 home", "typing")
            home.add_class("active")

            yield home
            yield NavItem(" settings", "settings")
            yield NavItem("󰋗 help", "help")
            yield NavItem(" about", "about")

        # yield NavItem("  " + get_username())



from textual.widget import Widget
from smassh.src import config_parser


class Ticker(Widget):
    """
    Ticker widget to show time/word left
    """

    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.update_check = self.set_interval(0.1, self.update)
        self.reset()

    def update(self) -> None:
        from smassh.ui.widgets import Space

        mode = config_parser.get("mode")
        stats = self.screen.query_one(Space).tracker.stats

        if not stats.start_time:
            return

        if mode == "words":
            count = config_parser.get("words_count")
            words_typed = stats.word_count
            self.text = f"{words_typed}/{count}"
        else:
            if stats.start_time:
                count = config_parser.get("time_count")
                time_remaining = count - stats.elapsed_time
                if time_remaining <= 0:
                    return self.screen.query_one(Space).finish_typing(fail=False)

                self.text = str(round(time_remaining))

        self.refresh()

    def reset(self) -> None:
        self.update_check.pause()
        mode = config_parser.get("mode")
        if mode == "words":
            count = config_parser.get("words_count")
            self.text = f"0/{count}"
        else:
            count = config_parser.get("time_count")
            self.text = str(count)

        self.refresh()

    def render(self) -> str:
        return self.text



from typing import Dict, Optional, Union
from rich.console import RenderableType
from rich.text import Span, Text
from textual.app import ComposeResult
from textual.widget import Widget

from smassh.ui.widgets.settings.danger_options import ResetConfig
from .option import Confirm, Option, NumberScroll

Options = Union[Option, NumberScroll, Confirm]


class SettingDescription(Widget):
    """
    Widget to show description of a setting
    """

    COMPONENT_CLASSES = {
        "setting--header",
        "setting--info",
        "setting--option",
        "setting--option-description",
        "setting--option-description-highlighted",
    }

    def __init__(
        self,
        title: str,
        info: str,
        items: Optional[Dict[str, str]] = None,
        *args,
        **kwargs,
    ) -> None:
        super().__init__(*args, **kwargs)
        self.title = title
        self.info = info
        self.items = items

    def render(self) -> RenderableType:
        def create_options(options: Dict[str, str]) -> Text:
            text = Text()
            for option, desc in options.items():
                desc = Text.from_markup(desc, style=c4)
                for index, span in enumerate(desc.spans):
                    if span.style == "green":
                        desc.spans[index] = Span(span.start, span.end, c5)

                text += Text(option, c3) + ": " + desc + "\n"

            return text

        c1 = self.get_component_rich_style("setting--header")
        c2 = self.get_component_rich_style("setting--info")
        c3 = self.get_component_rich_style("setting--option")
        c4 = self.get_component_rich_style("setting--option-description")
        c5 = self.get_component_rich_style("setting--option-description-highlighted")

        text = Text(self.title, c1) + "\n" + Text.from_markup(self.info, style=c2)
        if self.items:
            text = text + "\n" + create_options(self.items)

        return text


class Setting(Widget):
    """
    A class that holds one setting for the smassh!
    """

    DEFAULT_CSS = """
    Setting {
        layout: grid;
        grid-size: 2;
        grid-columns: 8fr 1fr;
        height: auto;
        border: blank;
    }
    """

    def __init__(
        self,
        title: str,
        items: dict[str, str],
        options: Options,
        info: str = "",
    ) -> None:
        super().__init__()
        self.title = title
        self.items = items
        self.options = options
        self.info = info
        if self.info:
            self.info += "\n"

    def set_section_widget(self, widget: Widget):
        self.section_widget = widget

    def select(self) -> None:
        self.options.highlight()

    def deselect(self) -> None:
        self.options.lowlight()

    def select_next(self) -> None:
        self.options.select_next_option()

    def select_prev(self) -> None:
        self.options.select_prev_option()

    def compose(self) -> ComposeResult:
        yield SettingDescription(self.title, self.info, self.items)
        yield self.options


# First menu
menu = {
    "behaviour": [
        Setting(
            " Min Speed",
            {},
            NumberScroll("min_speed"),
            info="Are you fast enough?"
            + "\n"
            + "Note: If your speed falls below this speed you will be declared failed",
        ),
        Setting(
            " Min Accuracy",
            {},
            NumberScroll("min_accuracy"),
            info="You can't go wrong with this"
            + "\n"
            + "Note: If your accuracy falls below this accuracy you will be declared failed",
        ),
        Setting(
            " Min Burst",
            {},
            NumberScroll("min_burst"),
            info="Wanna make your life harder?"
            + "\n"
            + "Note: If your speed for a word falls below this speed you will be declared failed",
        ),
        Setting(
            " Difficulty",
            {
                "normal": "You can type at your own accuracy",
                "expert": "Moving forward without writing the prev word correctly? YOU'RE FAILED!",
                "master": "A single incorrect press will declare you failed",
            },
            Option(
                "difficulty",
                options=["normal", "expert", "master"],
            ),
            "Where's the fun without some conditions?",
        ),
        Setting(
            "󰈉 Blind Mode",
            {
                "off": "You will get to know whether you typed right or  wrong",
                "on": "Just believe your spidey sense!",
            },
            Option("blind_mode", options=["off", "on"]),
            "Have a lot of Confidence? Try this !"
            + "\n"
            + "Note: You should turn [bold]force correct[/bold] off if you are turing blind mode on",
        ),
        Setting(
            " Tab Reset",
            {
                "on": "Pressing tab will cancel the current typing and re-start it",
                "off": "Pressing tab will have no effect ",
            },
            Option("tab_reset", options=["off", "on"]),
            "Lost your gusto in the middle of typing? Restart by hitting a tab!",
        ),
    ],
    "input": [
        Setting(
            " Force correct",
            {
                "on": "You will not be allowed to move forward until"
                + "\n"
                + "and unless all the characters until your cursor are typed correctly",
                "off": "You live your life your own way!",
            },
            Option("force_correct", options=["off", "on"]),
            "Are you worthy?",
        ),
        Setting(
            "󰁮 Confidence Mode:",
            {
                "off": "You can type backspace as many times you want",
                "on": "You will only be able to backspace until the start of the current word",
                "max": "You will not be able to press backspace at all",
            },
            Option("confidence_mode", options=["off", "on", "max"]),
            "Feeling Lucky?",
        ),
        Setting(
            "󰬵 Capitalization Mode:",
            {
                "off": "All lowercase words, don't worry about your 'Shift' key",
                "on": "You will come across some words that start with a capital letter, but not many",
                "max": "Try a mixed-case word trial",
            },
            Option(
                "capitalization_mode",
                options=["off", "on", "max"],
            ),
            "Getting your hands dirty?",
        ),
    ],
    "caret": [
        Setting(
            "󰗧 Caret style",
            {
                "off": "[green]hello pete[/green]r",
                "underline": "[green]hello pete[/green][underline]r[/underline]",
                "block": "[green]hello pete[/green][reverse]r[/reverse]",
            },
            Option(
                "caret_style",
                options=["off", "underline", "block"],
            ),
            info="Choose your style!",
        ),
        Setting(
            "󱔎 Cursor buddy",
            {},
            NumberScroll("cursor_buddy_speed"),
            info="Feeling a little lonely?"
            + "\n"
            + "A cursor will race along with you with this constant speed."
            + "\n"
            + "'0' means it will not be visible",
        ),
    ],
    "danger_zone": [
        Setting(
            "Reset Config",
            {},
            ResetConfig(),
            info="This will delete your current config and reset it to default",
        ),
    ],
    # "sound": [
    #     Setting(
    #         "Language",
    #         {},
    #         Option("language", options=["english", "french"]),
    #         "Choose the language you want the words to be generated from !",
    #     ),
    #     Setting(
    #         "Keypress Sound",
    #         {
    #             "off": "Already have good switches? There will be no sound on keypress",
    #             "on": "Pressing a key will trigger a click sound except backspace",
    #             "backspace": "Pressing key, including backspace, will trigger click sound",
    #         },
    #         Option(
    #             "keypress_sound",
    #             options=["off", "on", "backspace"],
    #         ),
    #         "Sounds good?",
    #     ),
    #     Setting(
    #         "Click sound",
    #         {
    #             "cream": "Smooth soothing sound to the ears :)",
    #             "lubed": "Just the right amount",
    #             "mech": "Mechanical feel baby!",
    #             "heavy": "Wanna feel like you are on a typewriter?",
    #         },
    #         Option(
    #             "sound",
    #             options=["cream", "lubed", "mech", "heavy"],
    #         ),
    #         "Choose whats most pleasing to you ears :)",
    #     ),
    # ],
}



from typing import Literal, Optional
from rich.console import RenderableType
from textual.app import ComposeResult
from textual.widget import Widget
from smassh.src import config_parser
from smassh.ui.widgets.typing.space import Space


class TypingStripItem(Widget):
    """
    Single setting widget for strip in the typing screen
    """

    DEFAULT_CSS = """
    TypingStripItem {
        width: auto;
        height: 1;
        padding: 0 1;
    }
    """

    def __init__(self, setting_name: str, icon: Optional[str] = None) -> None:
        super().__init__()
        self.setting_name = setting_name
        self.icon = icon
        self.shrink = False

    def render(self) -> RenderableType:
        if self.icon:
            return self.icon + " " + self.setting_name

        return self.setting_name


class Switchable(TypingStripItem):
    """
    A toggle-able setting for typing strip
    """

    setting_name: str
    setting_icon: str

    def __init__(self) -> None:
        super().__init__(self.setting_name, self.setting_icon)
        self.refresh_setting()

    def refresh_setting(self) -> None:
        self.set_class(config_parser.get(self.setting_name), "enabled")

    def _toggle(self) -> None: ...

    def toggle(self) -> None:
        self._toggle()
        self.refresh_setting()

    def on_click(self) -> None:
        self.toggle()
        self.screen.query_one(Space).reset()

    def render(self) -> RenderableType:
        self.refresh_setting()
        return super().render()


class PunctuationSwitch(Switchable):
    setting_name = "punctuations"
    setting_icon = "󰸥"

    def _toggle(self) -> None:
        config_parser.toggle_punctuations()


class NumberSwitch(Switchable):
    setting_name = "numbers"
    setting_icon = "󰲰"

    def _toggle(self) -> None:
        config_parser.toggle_numbers()


class WordMode(Switchable):
    setting_name = "words"
    setting_icon = "󰯬"

    def _refresh_mode_count(self) -> None:
        for counts in self.screen.query(ModeCount):
            counts.refresh()

    def _toggle(self) -> None:
        config_parser.toggle_mode()
        self._refresh_mode_count()
        self.screen.query_one(TimeMode).refresh_setting()

    def refresh_setting(self) -> None:
        configured_mode = config_parser.get("mode")
        self.set_class(configured_mode == "words", "enabled")


class TimeMode(Switchable):
    setting_name = "time"
    setting_icon = "󰥔"

    def _refresh_mode_count(self) -> None:
        for counts in self.screen.query(ModeCount):
            counts.refresh()

    def _toggle(self) -> None:
        config_parser.toggle_mode()
        self._refresh_mode_count()
        self.screen.query_one(WordMode).refresh_setting()

    def refresh_setting(self) -> None:
        configured_mode = config_parser.get("mode")
        self.set_class(configured_mode == "time", "enabled")


class ModeCount(Widget):
    DEFAULT_CSS = """
    ModeCount {
        width: auto;
    }
    """

    def __init__(self, value: int) -> None:
        super().__init__()
        self.value = value
        self.shrink = False

    def _refresh_other_counts(self) -> None:
        for counts in self.screen.query(ModeCount):
            counts.refresh()

    def on_click(self) -> None:
        config_parser.set(f"{config_parser.get('mode')}_count", self.value)
        self._refresh_other_counts()
        self.screen.query_one(Space).reset()

    def render(self) -> RenderableType:
        mode = config_parser.get("mode")
        count = config_parser.get(f"{mode}_count")
        self.set_class(count == self.value, "enabled")
        return str(self.value) + " "


class StripSeparator(Widget):
    DEFAULT_CSS = """
    StripSeparator {
        width: auto;
        height: 1;
    }
    """

    def __init__(self) -> None:
        super().__init__(disabled=True)

    def render(self) -> RenderableType:
        return " | "


class Bracket(Widget):
    DEFAULT_CSS = """
    Bracket {
        width: auto;
        height: 1;
    }
    """

    def __init__(self, bracket_type: Literal["left", "right"]) -> None:
        super().__init__(disabled=True)
        self.bracket_type = bracket_type

    def render(self) -> RenderableType:
        if self.bracket_type == "left":
            return ""
        else:
            return ""


class StripSection(Widget):
    DEFAULT_CSS = """
    StripSection {
        layout: horizontal;
        width: auto;
        height: 1;
    }
    """


class LeftStripSection(StripSection): ...


class MiddleStripSection(StripSection): ...


class RightStripSection(StripSection): ...


class TypingConfigStrip(Widget):
    DEFAULT_CSS = """
    TypingConfigStrip {
        column-span: 3;
        layout: horizontal;
        height: 3;
        align: center middle;
        padding: 0 1;
    }
    """

    def compose(self) -> ComposeResult:
        yield Bracket("left")

        with LeftStripSection():
            yield PunctuationSwitch()
            yield NumberSwitch()

        yield StripSeparator()

        with MiddleStripSection():
            yield WordMode()
            yield TimeMode()

        yield StripSeparator()

        with RightStripSection():
            for i in [15, 30, 60, 120]:
                yield ModeCount(i)

        yield Bracket("right")
